# -*- coding:utf-8 -*-
"""
Created on the 05/01/18
@author: Nicolas Thiebaut
@email: nkthiebaut@gmail.com
"""

OOV_TAG = "<oov>"  # Out-Of-Vocabulary tag for rare words

DEFAULT_PRETRAINED_EMBEDDINGS = {
    "fasttext": "fasttext-wiki-news-subwords-300",
    "word2vec": "word2vec-google-news-300",
    "glove": "glove-twitter-25",
}



# -*- coding:utf-8 -*-
"""
Created on the 05/01/18
@author: Nicolas Thiebaut
@email: nkthiebaut@gmail.com
"""
from pkg_resources import get_distribution

from .embeddings import EmbeddingTransformer  # noqa
from .texttransformers import RareWordsTagger, ItemSelector, TextStats  # noqa
from .keras_transformers import TextsToSequences, Padder  # noqa

__version__ = get_distribution("zeugma").version



# -*- coding:utf-8 -*-
"""
Created on the 05/01/18
@author: Nicolas Thiebaut
@email: nkthiebaut@gmail.com
"""
import os
from typing import Iterable, List, Union

import gensim.downloader as api
import numpy as np
from gensim.models.keyedvectors import Word2VecKeyedVectors
from sklearn.base import BaseEstimator, TransformerMixin
from zeugma.conf import DEFAULT_PRETRAINED_EMBEDDINGS
from zeugma.logger import package_logger as logger


class EmbeddingTransformer(BaseEstimator, TransformerMixin):
    """ Text vectorizer class: load pre-trained embeddings and transform texts
    into vectors.
    """

    model: Word2VecKeyedVectors
    aggregation: str

    def __init__(self, model: str = "glove", aggregation: str = "average"):
        """ Load pre-trained embeddings, either locally if model is a local file path
        or a Word2VecKeyedVector object, or downloaded from the gensim API if a string
        is provided.
        """
        if aggregation not in {"average", "sum", "minmax"}:
            raise ValueError(
                f"Unknown embeddings aggregation mode: {aggregation}, the available "
                "ones are: average, sum, or minmax."
            )
        if isinstance(model, str):
            model = model.lower()
            if model in DEFAULT_PRETRAINED_EMBEDDINGS.keys():
                model_gensim_name = DEFAULT_PRETRAINED_EMBEDDINGS[
                    model
                ]  # pragma: no cover
                self.model = api.load(model_gensim_name)  # pragma: no cover
            elif model in api.info()["models"].keys():
                self.model = api.load(model)
            elif os.path.exists(model):
                logger.info("Loading local model")
                self.model = Word2VecKeyedVectors.load(model)
                if not isinstance(self.model, Word2VecKeyedVectors):
                    raise TypeError(
                        "The input model should be a Word2VecKeyedVectors object but "
                        f"it is a {type(self.model)} object."
                    )
            else:
                raise KeyError(
                    f"Unknown pre-trained model name: {model}. Available models are"
                    + ", ".join(api.info()["models"].keys())
                )
            logger.info("Loaded model keyed vectors: " + model)  # pragma: no cover
        elif isinstance(model, Word2VecKeyedVectors):
            self.model = model
            logger.info("Loaded model keyed vectors.")
        else:
            raise TypeError(
                "Input pre-trained model should be a string or a gensim "
                "Word2VecKeyedVectors object"
            )
        self.aggregation = aggregation
        self.embedding_dimension = self.model.vector_size
        if self.aggregation == "minmax":
            self.embedding_dimension *= 2

    def transform_sentence(self, text: Union[Iterable, str]) -> np.array:
        """ Compute an aggregate embedding vector for an input str or iterable of str.
        """

        def preprocess_text(raw_text: Union[Iterable, str]) -> List[str]:
            """ Prepare text for the model, excluding unknown words"""
            if not isinstance(raw_text, list):
                if not isinstance(raw_text, str):
                    raise TypeError(
                        f"Input should be a str or a list of str, got {type(raw_text)}"
                    )
                raw_tokens = raw_text.split()
            return list(filter(lambda x: x in self.model.vocab, raw_tokens))

        tokens = preprocess_text(text)

        if not tokens:
            return np.zeros(self.embedding_dimension, dtype=np.float32)

        if self.aggregation == "average":
            return np.mean(self.model[tokens], axis=0)
        elif self.aggregation == "sum":
            return np.sum(self.model[tokens], axis=0)
        elif self.aggregation == "minmax":
            maxi = np.max(self.model[tokens], axis=0)
            mini = np.min(self.model[tokens], axis=0)
            return np.append(mini, maxi)

    def fit(self, x: Iterable[Iterable], y: Iterable = None) -> BaseEstimator:
        """ Has to define fit method to conform scikit-learn Transformer
        definition and integrate a sklearn.Pipeline object """
        return self  # pragma: no cover

    def transform(self, texts: Iterable[str]) -> Iterable[Iterable]:
        """ Transform corpus from single text transformation method """
        # TODO: parallelize this method with multiprocessing
        return np.array([self.transform_sentence(t) for t in texts])



# -*- coding:utf-8 -*-
"""
Created on the 02/05/2018
@author: Nicolas Thiebaut
@email: nicolas@visage.jobs
"""
import numpy as np
from keras.preprocessing.sequence import pad_sequences
from keras.preprocessing.text import Tokenizer
from sklearn.base import BaseEstimator
from sklearn.pipeline import TransformerMixin


class TextsToSequences(Tokenizer, BaseEstimator, TransformerMixin):
    """ Sklearn transformer to convert texts to indices list
    (e.g. [["the cute cat"], ["the dog"]] -> [[1, 2, 3], [1, 4]])"""

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    def fit(self, texts, y=None):
        self.fit_on_texts(texts)
        return self

    def transform(self, texts, y=None):
        return np.array(self.texts_to_sequences(texts))


class Padder(BaseEstimator, TransformerMixin):
    """ Pad and crop uneven lists to the same length.
    Only the end of lists longer than the max_length attribute are
    kept, and lists shorter than max_length are left-padded with zeros

    Attributes
    ----------
    max_length: int
        sizes of sequences after padding
    max_index: int
        maximum index known by the Padder, if a higher index is met during
        transform it is transformed to a 0
    """

    def __init__(self, max_length=500):
        self.max_length = max_length
        self.max_index = None

    def fit(self, X, y=None):
        self.max_index = pad_sequences(X, maxlen=self.max_length).max()
        return self

    def transform(self, X, y=None):
        X = pad_sequences(X, maxlen=self.max_length)
        X[X > self.max_index] = 0
        return X



# -*- coding:utf-8 -*-
"""
Created on the 01/25/2018
@author: Nicolas Thiebaut
@email: nicolas@visage.jobs
"""
from logging import WARNING, Formatter, StreamHandler, getLogger

LOG_FORMAT = "%(asctime)s [%(levelname)s]: %(message)s in %(pathname)s:%(lineno)d"

DEFAULT_LOG_LEVEL = WARNING


DEFAULT_HANDLER = StreamHandler()
DEFAULT_HANDLER.setFormatter(Formatter(LOG_FORMAT))

package_logger = getLogger("zeugma")
package_logger.setLevel(DEFAULT_LOG_LEVEL)
package_logger.addHandler(DEFAULT_HANDLER)



#!/usr/bin/env python

from setuptools import setup

setup(
    name="zeugma",
    packages=["zeugma"],
    version="0.46",
    python_requires=">=3.6",
    license="MIT",
    description="""Natural language processing (NLP) utils: word embeddings (Word2Vec,
    GloVe, FastText, ...) and preprocessing transformers, compatible with scikit-learn
    Pipelines.""",
    long_description=open("README.rst", "r").read(),
    long_description_content_type="text/x-rst",
    author="Nicolas Thiebaut",
    author_email="nkthiebaut@gmail.com",
    url="https://github.com/nkthiebaut",
    download_url="https://github.com/nkthiebaut/zeugma/archive/0.46.tar.gz",
    classifiers=[
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
    ],
    setup_requires=["pytest-runner", "numpy>=1.13.3", "Cython>=0.27.3"],
    install_requires=[
        "numpy>=1.13.3",
        "Cython>=0.27.3",
        "pandas>=0.20.3",
        "gensim>=3.5.0",
        "scikit_learn>=0.19.1",
        "tensorflow>=1.5.0",
        "keras>=2.1.3",
    ],
    tests_require=["pytest>=3.3.2"],
)
